@@ -1,8 +1,8 @@
 /************************************************************************
  * message.c      Message-handling routines for dopewars                *
- * Copyright (C)  1998-2005  Ben Webb                                   *
+ * Copyright (C)  1998-2020  Ben Webb                                   *
  *                Email: benwebb@users.sf.net                           *
- *                WWW: http://dopewars.sourceforge.net/                 *
+ *                WWW: https://dopewars.sourceforge.io/                 *
  *                                                                      *
  * This program is free software; you can redistribute it and/or        *
  * modify it under the terms of the GNU General Public License          *
@@ -145,14 +145,14 @@ void DoSendClientMessage(Player *From, AICode AI, MsgCode Code,
   text = g_string_new(NULL);
   if (HaveAbility(BufOwn, A_PLAYERID)) {
     if (To)
-      g_string_sprintfa(text, "%d", To->ID);
-    g_string_sprintfa(text, "^%c%c%s", AI, Code, Data ? Data : "");
+      g_string_append_printf(text, "%d", To->ID);
+    g_string_append_printf(text, "^%c%c%s", AI, Code, Data ? Data : "");
   } else {
-    g_string_sprintf(text, "%s^%s^%c%c%s", From ? GetPlayerName(From) : "",
+    g_string_printf(text, "%s^%s^%c%c%s", From ? GetPlayerName(From) : "",
                      To ? GetPlayerName(To) : "", AI, Code,
                      Data ? Data : "");
   }
-#if NETWORKING
+#ifdef NETWORKING
   if (!Network) {
 #endif
     if (From)
@@ -164,7 +164,7 @@ void DoSendClientMessage(Player *From, AICode AI, MsgCode Code,
       FirstServer = AddPlayer(0, ServerFrom, FirstServer);
     }
     HandleServerMessage(text->str, ServerFrom);
-#if NETWORKING
+#ifdef NETWORKING
   } else {
     QueuePlayerMessageForSend(BufOwn, text->str);
   }
@@ -206,19 +206,19 @@ void SendServerMessage(Player *From, AICode AI, MsgCode Code,
   text = g_string_new(NULL);
   if (HaveAbility(To, A_PLAYERID)) {
     if (From)
-      g_string_sprintfa(text, "%d", From->ID);
-    g_string_sprintfa(text, "^%c%c%s", AI, Code, Data ? Data : "");
+      g_string_append_printf(text, "%d", From->ID);
+    g_string_append_printf(text, "^%c%c%s", AI, Code, Data ? Data : "");
   } else {
-    g_string_sprintf(text, "%s^%s^%c%c%s", From ? GetPlayerName(From) : "",
+    g_string_printf(text, "%s^%s^%c%c%s", From ? GetPlayerName(From) : "",
                      To ? GetPlayerName(To) : "", AI, Code,
                      Data ? Data : "");
   }
-#if NETWORKING
+#ifdef NETWORKING
   if (!Network) {
 #endif
     if (ClientMessageHandlerPt)
       (*ClientMessageHandlerPt)(text->str, (Player *)(FirstClient->data));
-#if NETWORKING
+#ifdef NETWORKING
   } else {
     QueuePlayerMessageForSend(To, text->str);
   }
@@ -251,11 +251,7 @@ void InitAbilities(Player *Play)
   Play->Abil.Local[A_DRUGVALUE] = (DrugValue ? TRUE : FALSE);
   Play->Abil.Local[A_TSTRING] = TRUE;
   Play->Abil.Local[A_DONEFIGHT] = TRUE;
-#ifdef HAVE_GLIB2
   Play->Abil.Local[A_UTF8] = TRUE;
-#else
-  Play->Abil.Local[A_UTF8] = FALSE;
-#endif
   Play->Abil.Local[A_DATE] = TRUE;
 
   if (!Network) {
@@ -353,14 +349,15 @@ gboolean HaveAbility(Player *Play, gint Type)
     return (Play->Abil.Shared[Type]);
 }
 
-#if NETWORKING
+#ifdef NETWORKING
 /* 
  * Reads and writes player data from/to the network if it is ready.
  * If any data were read, TRUE is returned. "DoneOK" is set TRUE
  * unless a fatal error (i.e. the connection was broken) occurred.
  */
 gboolean PlayerHandleNetwork(Player *Play, gboolean ReadReady,
-                             gboolean WriteReady, gboolean *DoneOK)
+                             gboolean WriteReady, gboolean ErrorReady,
+                             gboolean *DoneOK)
 {
   gboolean DataWaiting = FALSE;
 
@@ -368,7 +365,8 @@ gboolean PlayerHandleNetwork(Player *Play, gboolean ReadReady,
   if (!Play)
     return DataWaiting;
   DataWaiting =
-      NetBufHandleNetwork(&Play->NetBuf, ReadReady, WriteReady, DoneOK);
+      NetBufHandleNetwork(&Play->NetBuf, ReadReady, WriteReady, ErrorReady,
+                          DoneOK);
 
   return DataWaiting;
 }
@@ -408,103 +406,72 @@ gboolean WritePlayerDataToWire(Player *Play)
   return WriteDataToWire(&Play->NetBuf);
 }
 
-typedef enum {
-  MEC_INTERNAL,
-  MEC_BADREPLY
-} MetaErrorCode;
-
-static void MetaAppendError(GString *str, LastError *error)
+gboolean OpenMetaHttpConnection(CurlConnection *conn, GError **err)
 {
-  switch (error->code) {
-  case MEC_INTERNAL:
-    g_string_sprintfa(str, _("Internal metaserver error \"%s\""),
-                      (gchar *)error->data);
-    break;
-  case MEC_BADREPLY:
-    g_string_sprintfa(str, _("Bad metaserver reply \"%s\""),
-                      (gchar *)error->data);
-    break;
-  default:
-    g_string_sprintfa(str, _("Unknown metaserver error code %d"),
-                      error->code);
-    break;
+  gboolean ret;
+  gchar *url;
+
+  url = g_strdup_printf("%s?output=text&getlist=%d",
+                        MetaServer.URL, METAVERSION);
+  ret = OpenCurlConnection(conn, url, NULL, err);
+  g_free(url);
+  return ret;
+}
+
+GQuark dope_meta_error_quark(void)
+{
+  return g_quark_from_static_string("dope-meta-error-quark");
+}
+
+gboolean HandleWaitingMetaServerData(CurlConnection *conn, GSList **listpt,
+                                     GError **err)
+{
+  char *msg;
+
+  g_assert(conn && listpt);
+
+  msg = conn->data;
+  /* This should be the first line of the body, the "MetaServer:" line */
+  if (msg && strlen(msg) >= 14 && strncmp(msg, "FATAL ERROR:", 12) == 0) {
+    g_set_error(err, DOPE_META_ERROR, DOPE_META_ERROR_INTERNAL,
+                _("Internal metaserver error \"%s\""), &msg[13]);
+    return FALSE;
+  } else if (msg && strncmp(msg, "MetaServer:", 11) != 0) {
+    g_set_error(err, DOPE_META_ERROR, DOPE_META_ERROR_BAD_REPLY,
+                _("Bad metaserver reply \"%s\""), msg);
+    return FALSE;
   }
-}
 
-static ErrorType ETMeta = { MetaAppendError, NULL };
-
-gboolean OpenMetaHttpConnection(HttpConnection **conn)
-{
-  gchar *query;
-  gboolean retval;
-
-  query = g_strdup_printf("%s?output=text&getlist=%d",
-                          MetaServer.Path, METAVERSION);
-  retval = OpenHttpConnection(conn, MetaServer.Name, MetaServer.Port,
-                              MetaServer.ProxyName, MetaServer.ProxyPort,
-                              "",
-                              UseSocks
-                              && MetaServer.UseSocks ? &Socks : NULL,
-                              "GET", query, NULL, NULL);
-  g_free(query);
-  return retval;
-}
-
-gboolean HandleWaitingMetaServerData(HttpConnection *conn,
-                                     GSList **listpt, gboolean *doneOK)
-{
-  gchar *msg;
-  ServerData *NewServer;
-
-  g_assert(conn && listpt && doneOK);
-
-  /* If we're done reading the headers, only read if the data for a whole
-   * server is available (8 lines) N.B. "Status" is from the _last_ read */
-  if (conn->Status == HS_READBODY && conn->StatusCode == 200) {
-    if (CountWaitingMessages(&conn->NetBuf) < 8)
-      return FALSE;
-
-    NewServer = g_new0(ServerData, 1);
-    NewServer->Name = ReadHttpResponse(conn, doneOK);
-    msg = ReadHttpResponse(conn, doneOK);
-    NewServer->Port = atoi(msg);
-    g_free(msg);
-    NewServer->Version = ReadHttpResponse(conn, doneOK);
-    msg = ReadHttpResponse(conn, doneOK);
-    if (msg[0])
-      NewServer->CurPlayers = atoi(msg);
-    else
-      NewServer->CurPlayers = -1;
-    g_free(msg);
-    msg = ReadHttpResponse(conn, doneOK);
-    NewServer->MaxPlayers = atoi(msg);
-    g_free(msg);
-    NewServer->Update = ReadHttpResponse(conn, doneOK);
-    NewServer->Comment = ReadHttpResponse(conn, doneOK);
-    NewServer->UpSince = ReadHttpResponse(conn, doneOK);
-    *listpt = g_slist_append(*listpt, NewServer);
-  } else if (conn->Status == HS_READSEPARATOR && conn->StatusCode == 200) {
-    /* This should be the first line of the body, the "MetaServer:" line */
-    msg = ReadHttpResponse(conn, doneOK);
-    if (!msg)
-      return FALSE;
-    if (strlen(msg) >= 14 && strncmp(msg, "FATAL ERROR:", 12) == 0) {
-      SetError(&conn->NetBuf.error, &ETMeta, MEC_INTERNAL,
-               g_strdup(&msg[13]));
-      *doneOK = FALSE;
-      return FALSE;
-    } else if (strncmp(msg, "MetaServer:", 11) != 0) {
-      SetError(&conn->NetBuf.error, &ETMeta, MEC_BADREPLY, g_strdup(msg));
-      g_free(msg);
-      *doneOK = FALSE;
-      return FALSE;
+  msg = CurlNextLine(conn, msg);
+  while (msg) {
+    char *name, *port, *version, *curplayers, *maxplayers, *update,
+	 *comment, *upsince;
+    name = msg;
+    port = CurlNextLine(conn, name);
+    version = CurlNextLine(conn, port);
+    curplayers = CurlNextLine(conn, version);
+    maxplayers = CurlNextLine(conn, curplayers);
+    update = CurlNextLine(conn, maxplayers);
+    comment = CurlNextLine(conn, update);
+    upsince = CurlNextLine(conn, comment);
+    msg = CurlNextLine(conn, upsince);
+    if (msg) {
+      ServerData *NewServer = g_new0(ServerData, 1);
+      NewServer->Name = g_strdup(name);
+      NewServer->Port = atoi(port);
+      NewServer->Version = g_strdup(version);
+      NewServer->CurPlayers = curplayers[0] ? atoi(curplayers) : -1;
+      NewServer->MaxPlayers = atoi(maxplayers);
+      NewServer->Update = g_strdup(update);
+      NewServer->Comment = g_strdup(comment);
+      NewServer->UpSince = g_strdup(upsince);
+      *listpt = g_slist_append(*listpt, NewServer);
     }
-    g_free(msg);
-  } else {
-    msg = ReadHttpResponse(conn, doneOK);
-    if (!msg)
-      return FALSE;
-    g_free(msg);
+  }
+  if (!*listpt) {
+    g_set_error_literal(err, DOPE_META_ERROR, DOPE_META_ERROR_EMPTY,
+                        _("No servers listed on metaserver"));
+    return FALSE;
   }
   return TRUE;
 }
@@ -558,7 +525,7 @@ void AddURLEnc(GString *str, gchar *unenc)
     } else if (unenc[i] == ' ') {
       g_string_append_c(str, '+');
     } else {
-      g_string_sprintfa(str, "%%%02X", unenc[i]);
+      g_string_append_printf(str, "%%%02X", unenc[i]);
     }
   }
 }
@@ -594,10 +561,10 @@ void SendInventory(Player *From, AICode AI, MsgCode Code,
 
   text = g_string_new(NULL);
   for (i = 0; i < NumGun; i++) {
-    g_string_sprintfa(text, "%d:", Guns ? Guns[i].Carried : 0);
+    g_string_append_printf(text, "%d:", Guns ? Guns[i].Carried : 0);
   }
   for (i = 0; i < NumDrug; i++) {
-    g_string_sprintfa(text, "%d:", Drugs ? Drugs[i].Carried : 0);
+    g_string_append_printf(text, "%d:", Drugs ? Drugs[i].Carried : 0);
   }
   SendServerMessage(From, AI, Code, To, text->str);
   g_string_free(text, TRUE);
@@ -645,7 +612,7 @@ void SendSpyReport(Player *To, Player *SpiedOn)
   int i;
 
   text = g_string_new(NULL);
-  g_string_sprintf(text, "%s^%s^%s^%d^%d^%d^%d^%d^",
+  g_string_printf(text, "%s^%s^%s^%d^%d^%d^%d^%d^",
                    (cashstr = pricetostr(SpiedOn->Cash)),
                    (debtstr = pricetostr(SpiedOn->Debt)),
                    (bankstr = pricetostr(SpiedOn->Bank)),
@@ -655,23 +622,24 @@ void SendSpyReport(Player *To, Player *SpiedOn)
   g_free(debtstr);
   g_free(bankstr);
   if (HaveAbility(SpiedOn, A_DATE)) {
-    g_string_sprintfa(text, "%d^%d^%d^", g_date_day(SpiedOn->date),
-                      g_date_month(SpiedOn->date), g_date_year(SpiedOn->date));
+    g_string_append_printf(text, "%d^%d^%d^", g_date_get_day(SpiedOn->date),
+                      g_date_get_month(SpiedOn->date),
+                      g_date_get_year(SpiedOn->date));
   }
   for (i = 0; i < NumGun; i++) {
-    g_string_sprintfa(text, "%d^", SpiedOn->Guns[i].Carried);
+    g_string_append_printf(text, "%d^", SpiedOn->Guns[i].Carried);
   }
   for (i = 0; i < NumDrug; i++) {
-    g_string_sprintfa(text, "%d^", SpiedOn->Drugs[i].Carried);
+    g_string_append_printf(text, "%d^", SpiedOn->Drugs[i].Carried);
   }
   if (HaveAbility(To, A_DRUGVALUE))
     for (i = 0; i < NumDrug; i++) {
-      g_string_sprintfa(text, "%s^",
+      g_string_append_printf(text, "%s^",
                         (cashstr =
                          pricetostr(SpiedOn->Drugs[i].TotalValue)));
       g_free(cashstr);
     }
-  g_string_sprintfa(text, "%d", SpiedOn->Bitches.Carried);
+  g_string_append_printf(text, "%d", SpiedOn->Bitches.Carried);
   if (To != SpiedOn)
     SendServerMessage(SpiedOn, C_NONE, C_UPDATE, To, text->str);
   else
@@ -699,7 +667,7 @@ void SendInitialData(Player *To)
       LocalNames[i] = GetDefaultTString(LocalNames[i]);
     }
   text = g_string_new("");
-  g_string_sprintf(text, "%s^%d^%d^%d^", VERSION, NumLocation, NumGun,
+  g_string_printf(text, "%s^%d^%d^%d^", VERSION, NumLocation, NumGun,
                    NumDrug);
   for (i = 0; i < 6; i++) {
     g_string_append(text, LocalNames[i]);
@@ -710,11 +678,11 @@ void SendInitialData(Player *To)
     g_string_append(text, LocalNames[6]);
     g_string_append_c(text, '^');
   } else {
-    g_string_sprintfa(text, "%d-^-%d^", StartDate.month, StartDate.year);
+    g_string_append_printf(text, "%d-^-%d^", StartDate.month, StartDate.year);
   }
 
   if (HaveAbility(To, A_PLAYERID))
-    g_string_sprintfa(text, "%d^", To->ID);
+    g_string_append_printf(text, "%d^", To->ID);
 
   /* Player ID is expected after the first 7 names, so send the rest now */
   for (i = 7; i < NUMNAMES; i++) {
@@ -727,7 +695,7 @@ void SendInitialData(Player *To)
       g_free(LocalNames[i]);
     }
 
-  g_string_sprintfa(text, "%c%s^", Currency.Prefix ? '1' : '0',
+  g_string_append_printf(text, "%c%s^", Currency.Prefix ? '1' : '0',
                     Currency.Symbol);
   SendServerMessage(NULL, C_NONE, C_INIT, To, text->str);
   g_string_free(text, TRUE);
@@ -735,11 +703,11 @@ void SendInitialData(Player *To)
 
 void ReceiveInitialData(Player *Play, char *Data)
 {
-  char *pt, *ServerVersion, *curr;
+  char *pt, *curr;
   GSList *list;
 
   pt = Data;
-  ServerVersion = GetNextWord(&pt, "(unknown)");
+  GetNextWord(&pt, "(unknown)"); /* server version */
   ResizeLocations(GetNextInt(&pt, NumLocation));
   ResizeGuns(GetNextInt(&pt, NumGun));
   ResizeDrugs(GetNextInt(&pt, NumDrug));
@@ -1305,7 +1273,7 @@ void SendFightMessage(Player *Attacker, Player *Defender,
         }
       } else
         BitchName = "";
-      g_string_sprintf(text, "%s^%s^%d^%d^%s^%d^%d^%c%c%c%c^",
+      g_string_printf(text, "%s^%s^%d^%d^%s^%d^%d^%c%c%c%c^",
                        Attacker == To ? "" : GetPlayerName(Attacker),
                        (Defender == To || Defender == NULL)
                        ? "" : GetPlayerName(Defender),
@@ -1373,29 +1341,29 @@ void FormatFightMessage(Player *To, GString *text, Player *Attacker,
     if (DefendName[0]) {
       if (IsCop(Defender) && !AttackName[0]) {
         if (Bitches == 0) {
-          dpg_string_sprintfa(text, _("%s - %s - is chasing you, man!"),
+          dpg_string_append_printf(text, _("%s - %s - is chasing you, man!"),
                               DefendName, Armament);
         } else {
-          dpg_string_sprintfa(text,
+          dpg_string_append_printf(text,
                               _("%s and %d %tde - %s - are chasing you, man!"),
                               DefendName, Bitches, BitchesName, Armament);
         }
       } else {
-        dpg_string_sprintfa(text, _("%s arrives with %d %tde, %s!"),
+        dpg_string_append_printf(text, _("%s arrives with %d %tde, %s!"),
                             DefendName, Bitches, BitchesName, Armament);
       }
     }
     break;
   case F_STAND:
     if (AttackName[0]) {
-      g_string_sprintfa(text, _("%s stands and takes it"), AttackName);
+      g_string_append_printf(text, _("%s stands and takes it"), AttackName);
     } else {
       g_string_append(text, _("You stand there like a dummy."));
     }
     break;
   case F_FAILFLEE:
     if (AttackName[0]) {
-      g_string_sprintfa(text, _("%s tries to get away, but fails."),
+      g_string_append_printf(text, _("%s tries to get away, but fails."),
                         AttackName);
     } else {
       g_string_append(text, _("Panic! You can't get away!"));
@@ -1407,13 +1375,13 @@ void FormatFightMessage(Player *To, GString *text, Player *Attacker,
       if (AttackName[0]) {
         if (!IsCop(Attacker) && brandom(0, 100) < 70
             && Attacker->IsAt >= 0) {
-          dpg_string_sprintfa(text, _("%s has got away to %tde!"), AttackName,
+          dpg_string_append_printf(text, _("%s has got away to %tde!"), AttackName,
                               Location[Attacker->IsAt].Name);
         } else {
-          g_string_sprintfa(text, _("%s has got away!"), AttackName);
+          g_string_append_printf(text, _("%s has got away!"), AttackName);
         }
       } else {
-        g_string_sprintfa(text, _("You got away!"));
+        g_string_append_printf(text, _("You got away!"));
       }
     }
     break;
@@ -1424,49 +1392,49 @@ void FormatFightMessage(Player *To, GString *text, Player *Attacker,
     break;
   case F_MISS:
     if (AttackName[0] && DefendName[0]) {
-      g_string_sprintfa(text, _("%s shoots at %s... and misses!"),
+      g_string_append_printf(text, _("%s shoots at %s... and misses!"),
                         AttackName, DefendName);
     } else if (AttackName[0]) {
-      g_string_sprintfa(text, _("%s shoots at you... and misses!"),
+      g_string_append_printf(text, _("%s shoots at you... and misses!"),
                         AttackName);
     } else if (DefendName[0]) {
-      g_string_sprintfa(text, _("You missed %s!"), DefendName);
+      g_string_append_printf(text, _("You missed %s!"), DefendName);
     }
     break;
   case F_HIT:
     if (AttackName[0] && DefendName[0]) {
       if (Health == 0 && Bitches == 0) {
-        g_string_sprintfa(text, _("%s shoots %s dead."),
+        g_string_append_printf(text, _("%s shoots %s dead."),
                           AttackName, DefendName);
       } else if (BitchesKilled) {
-        dpg_string_sprintfa(text, _("%s shoots at %s and kills a %tde!"),
+        dpg_string_append_printf(text, _("%s shoots at %s and kills a %tde!"),
                             AttackName, DefendName, BitchName);
       } else {
-        g_string_sprintfa(text, _("%s shoots at %s."),
+        g_string_append_printf(text, _("%s shoots at %s."),
                           AttackName, DefendName);
       }
     } else if (AttackName[0]) {
       if (Health == 0 && Bitches == 0) {
-        g_string_sprintfa(text, _("%s wasted you, man! What a drag!"),
+        g_string_append_printf(text, _("%s wasted you, man! What a drag!"),
                           AttackName);
       } else if (BitchesKilled) {
-        dpg_string_sprintfa(text,
+        dpg_string_append_printf(text,
                             _("%s shoots at you... and kills a %tde!"),
                             AttackName, BitchName);
       } else {
-        g_string_sprintfa(text, _("%s hits you, man!"), AttackName);
+        g_string_append_printf(text, _("%s hits you, man!"), AttackName);
       }
     } else if (DefendName[0]) {
       if (Health == 0 && Bitches == 0) {
-        g_string_sprintfa(text, _("You killed %s!"), DefendName);
+        g_string_append_printf(text, _("You killed %s!"), DefendName);
       } else if (BitchesKilled) {
-        dpg_string_sprintfa(text, _("You hit %s, and killed a %tde!"),
+        dpg_string_append_printf(text, _("You hit %s, and killed a %tde!"),
                             DefendName, BitchName);
       } else {
-        g_string_sprintfa(text, _("You hit %s!"), DefendName);
+        g_string_append_printf(text, _("You hit %s!"), DefendName);
       }
       if (Loot > 0) {
-        dpg_string_sprintfa(text, _(" You find %P on the body!"), Loot);
+        dpg_string_append_printf(text, _(" You find %P on the body!"), Loot);
       } else if (Loot < 0) {
         g_string_append(text, _(" You loot the body!"));
       }
