@@ -1,8 +1,8 @@
 /************************************************************************
  * util.h         Miscellaneous utility and portability functions       *
- * Copyright (C)  1998-2005  Ben Webb                                   *
+ * Copyright (C)  1998-2020  Ben Webb                                   *
  *                Email: benwebb@users.sf.net                           *
- *                WWW: http://dopewars.sourceforge.net/                 *
+ *                WWW: https://dopewars.sourceforge.io/                 *
  *                                                                      *
  * This program is free software; you can redistribute it and/or        *
  * modify it under the terms of the GNU General Public License          *
@@ -30,6 +30,7 @@
 #include <stdio.h>
 
 #ifdef CYGWIN                   /* Definitions for native Win32 build */
+#include <winsock2.h>
 #include <windows.h>
 #include <string.h>
 
