@@ -1,8 +1,8 @@
 /************************************************************************
  * winmain.h      Startup code and support for the Win32 platform       *
- * Copyright (C)  1998-2005  Ben Webb                                   *
+ * Copyright (C)  1998-2020  Ben Webb                                   *
  *                Email: benwebb@users.sf.net                           *
- *                WWW: http://dopewars.sourceforge.net/                 *
+ *                WWW: https://dopewars.sourceforge.io/                 *
  *                                                                      *
  * This program is free software; you can redistribute it and/or        *
  * modify it under the terms of the GNU General Public License          *
@@ -25,6 +25,8 @@
 
 #ifdef CYGWIN
 
+#include <glib.h>
+
 gchar *GetBinaryDir(void);
 
 #endif /* CYGWIN */
